$ifndef _CDC_DESC_INC_
$define _CDC_DESC_INC_
'
' USB descriptors for a CDC device
'
$define cVID                    0x04D8     ' Vendor ID (Keep to 0x format for Hex value)
$define cPID                    0x000A     ' Product ID (Keep to 0x format for Hex value)
$define cUSB_EP0_BUFF_SIZE      8          ' 8, 16, 32, or 64
$define cUSB_MAX_NUM_INT        2
$define cUSB_MAX_EP_NUMBER      3
$define cUSB_NUM_CONFIGS        1
$define cUSB_NUM_INTERFACES     2
$define cUSB_MAX_POWER          100        ' Maximum power consumption (in MilliAmps)
$define cUSB_POLLING_INTERVAL   2          ' Polling interval (in MilliSeconds)

'$define cUSE_SELF_POWER                 ' Self powered?
'$define cUSE_USB_BUS_SENSE_IO

$ifdef cUSE_USB_BUS_SENSE_IO
    Symbol USB_BUS_SENSE_PIN = PORTB.0
$endif

$define cUSB_POLLING

' Endpoints Allocation
$define cCDC_COMM_INTF_ID     0
$define cCDC_COMM_EP          2
$define cCDC_COMM_IN_EP_SIZE  8
$define cCDC_DATA_INTF_ID     1
$define cCDC_DATA_EP          3
$define cCDC_DATA_OUT_EP_SIZE 64
$define cCDC_DATA_IN_EP_SIZE  64

Symbol cUSB_BUFFER_OUT_SIZE = cCDC_DATA_OUT_EP_SIZE
Symbol cUSB_BUFFER_IN_SIZE  = cCDC_DATA_IN_EP_SIZE

    GoTo _DescriptorMain_               ' Jump over the USB code
    Include "USB_CDC.inc"               ' Include the  CDC class code
'
'--------------------------------------------------------------------------------------------------------------------
' This table is polled by the host immediately after USB Reset has been released.
' This table defines the maximum packet size EP0 can take.
' See section 9.6.1 of the Rev 1.0 USB specification.
' These fields are application dependent. Modify these to meet your specifications.
'
Desc_DeviceDescriptor:
    Dt (Desc_EndDeviceDescriptor - Desc_DeviceDescriptor) / 2 ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_DEVICE           ' Device descriptor type
    Dt 0x10, 0x01                       ' USB Spec Release Number in BCD format - 1.10 (low byte, high byte)
    Dt cCDC_DEVICE                      ' Class Code
    Dt 0x00                             ' Subclass code
    Dt 0x00                             ' Protocol code
    Dt cUSB_EP0_BUFF_SIZE               ' Max packet size for EP0
    Dt Low(cVID), High(cVID)            ' Vendor ID - 0x04D8 is Microchip Vendor ID (low byte, high byte)
    Dt Low(cPID), High(cPID)            ' Product ID: CDC RS - 232 Emulation Demo (low byte, high byte)
    Dt 0x01, 0x00                       ' Device release number in BCD format (low byte, high byte)
    Dt 0x01                             ' Manufacturer string index
    Dt 0x02                             ' Product string index
    Dt 0x00                             ' Device serial number string index
    Dt cUSB_NUM_CONFIGS                 ' Number of possible configurations
Desc_EndDeviceDescriptor:
'
'--------------------------------------------------------------------------------------------------------------------
' This table is retrieved by the host after the address has been set.
' This table defines the configurations available for the device.
' See section 9.6.2 of the Rev 1.0 USB specification (page 184).
' These fields are application dependant.
' Modify these to meet your specifications.
'
' Configuration Descriptor
Desc_Config1:
    Dt (Desc_Interface1 - Desc_Config1) / 2         ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_CONFIGURATION                ' Configuration descriptor type

Desc_Config1Len:
    Dt ((Desc_EndConfig1 - Desc_Config1) / 2), 0x00 ' Total length of data for this cfg
    Dt cUSB_NUM_INTERFACES                          ' Number of interfaces in this cfg
    Dt 0x01                                         ' Index value of this configuration
    Dt 0x00                                         ' Configuration string index
    Dt _cDEFAULT | _cSELF                           ' Attributes
    Dt (cUSB_MAX_POWER * 2)                         ' Max power consumption (in mA)

Desc_Interface1:
    Dt (Desc_CDCDescriptor1 - Desc_Interface1) / 2  ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_INTERFACE                    ' Interface descriptor type
    Dt 0x00                                         ' Interface Number
    Dt 0x00                                         ' Alternate Setting Number
    Dt 0x01                                         ' Number of endpoints in this intf
    Dt cCOMM_INTF                                   ' Class code
    Dt cABSTRACT_CONTROL_MODEL                      ' Subclass code
    Dt cV25TER                                      ' Protocol code
    Dt 0x00                                         ' Interface string index
Desc_CDCDescriptor1:
    Dt (Desc_CDCDescriptor2 - Desc_CDCDescriptor1) / 2 ' Size of this descriptor (in bytes)
    Dt cCS_INTERFACE                                ' Descriptor type
    Dt cDSC_FN_HEADER                               ' Descriptor sub type
    Dt 0x10, 0x01                                   ' (low byte, high byte)

Desc_CDCDescriptor2:
    Dt (Desc_CDCDescriptor3 - Desc_CDCDescriptor2) / 2 ' Size of this descriptor (in bytes)
    Dt cCS_INTERFACE                                ' Descriptor type
    Dt cDSC_FN_ACM                                  ' Descriptor sub type
    Dt 0x02                                         ' USB_CDC_ACM_FN_DSC_VAL, Capabilities

Desc_CDCDescriptor3:
    Dt (Desc_CDCDescriptor4 - Desc_CDCDescriptor3) / 2 ' Size of this descriptor (in bytes)
    Dt cCS_INTERFACE                                ' Descriptor type
    Dt cDSC_FN_UNION                                ' Descriptor sub type
    Dt cCDC_COMM_INTF_ID                            ' Master interface
    Dt cCDC_DATA_INTF_ID                            ' Slave interface

Desc_CDCDescriptor4:
    Dt (Desc_Endpoint2 - Desc_CDCDescriptor4) / 2   ' Size of this descriptor (in bytes)
    Dt cCS_INTERFACE                                ' Descriptor type
    Dt cDSC_FN_CALL_MGT                             ' Descriptor sub type
    Dt 0x00                                         ' Capabilities
    Dt cCDC_DATA_INTF_ID                            ' Data interface

Desc_Endpoint2:
    Dt (Desc_Interface2 - Desc_Endpoint2) / 2       ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_ENDPOINT                     ' EndPoint descriptor type
    Dt cCDC_COMM_EP | _cEP_IN                       ' _EP02_IN, Endpoint Address
    Dt _cINT                                        ' Attributes
    Dt Low(cCDC_COMM_IN_EP_SIZE), High(cCDC_COMM_IN_EP_SIZE)  ' Size of the endpoint buffer (in bytes) (low byte, high byte)
    Dt cUSB_POLLING_INTERVAL                        ' Polling interval (in ms)

Desc_Interface2:
    Dt (Desc_Endpoint3Out - Desc_Interface2) / 2    ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_INTERFACE                    ' Interface descriptor type
    Dt 0x01                                         ' Interface Number
    Dt 0x00                                         ' Alternate Setting Number
    Dt 0x02                                         ' Number of endpoints in this intf
    Dt cDATA_INTF                                   ' Class code
    Dt 0x00                                         ' Subclass code
    Dt cNO_PROTOCOL                                 ' Protocol code
    Dt 0x00                                         ' Interface string index

Desc_Endpoint3Out:
    Dt (Desc_Endpoint3In - Desc_Endpoint3Out) / 2   ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_ENDPOINT                     ' EndPoint descriptor type
    Dt cCDC_DATA_EP | _cEP_OUT                      ' _EP03_OUT, Endpoint Address
    Dt _cBULK                                       ' Attributes
    Dt Low(cCDC_DATA_OUT_EP_SIZE), High(cCDC_DATA_OUT_EP_SIZE)  ' Size of the endpoint buffer (in bytes) (low byte, high byte)
    Dt 0x00                                         ' Polling interval

Desc_Endpoint3In:
    Dt (Desc_EndConfig1 - Desc_Endpoint3In) / 2     ' Size of this descriptor (in bytes)
    Dt cUSB_DESCRIPTOR_ENDPOINT                     ' EndPoint descriptor type
    Dt cCDC_DATA_EP | _cEP_IN                       ' _EP03_IN, Endpoint Address
    Dt _cBULK                                       ' Attributes
    Dt Low(cCDC_DATA_IN_EP_SIZE), High(cCDC_DATA_IN_EP_SIZE)  ' Size of the endpoint buffer (in bytes) (low byte, High Byte)
    Dt 0x00                                         ' Polling interval
Desc_EndConfig1:
'
' Configuration pointer table
'
Desc_USB_CD_Ptr:
Desc_Configs:
    CData As Dword Desc_Config1
'
' String pointer table
'
Desc_USB_SD_Ptr:
Desc_Strings:
    CData As Dword Desc_String0, Desc_String1, Desc_String2
'
' Language code string descriptor
'
Desc_String0:-
    dt (Desc_String1 - Desc_String0) / 2            ' Size of this descriptor (in bytes)
    dt cUSB_DESCRIPTOR_STRING                       ' String descriptor type
    dt 0x09, 0x04                                   ' Language ID as defined by MS - 0x0409
'
' Manufacturer string descriptor
'
Desc_String1:-
    dt (Desc_String2 - Desc_String1) / 2            ' Size of this descriptor (in bytes)
    dt cUSB_DESCRIPTOR_STRING                       ' String descriptor type
    CData As Dword "Positron"
'
' Product string descriptor
'
Desc_String2:-
    dt (_DescriptorMain_ - Desc_String2) / 2  ' Size of this descriptor (in bytes)
    dt cUSB_DESCRIPTOR_STRING                       ' String descriptor type
    CData As Dword "CDC RS-232 Emulation Demo"

_DescriptorMain_:
$endif  ' _CDC_DESC_INC_
